--[[  **** SCRIPT DESCRIPTION ****
********************************************************************
Written by Ken Fischer for the Cedar Driver installation.
This script is brand neutral

Last Modified: 1-27-2011
Last Modified by: K Fischer

Script to determine if a dialog is displayed offering to install the
XPS Card Printer Plug-in. This is only offered if the following is true:

(Version 5.0 or newer of the Print Server component in installed) AND
((XPS Card Printer Plug-in not installed) OR (Plug-in installed but build number is older))

Version of plug-in contained on install media is read from the DXP01Install.ini file in the Autoplay
folder.  The section is [Plugin] and the ini variable name is Build.

Variables available to the calling lua script. These will be set in this script.
--> bDisplayPluginInstallDialog: boolean telling installation to offer plug-in install
--> bUpdateMode: boolean telling application that this will be an update.  Don't know if this will be used yet.

************************************************************************* 
--]]

--Initialize local script variables
local PrintServerDLL = ""; -- string to hold the full path to the print server dll
local LastError = 0;
local PrintServerPath = "";
bDisplayPluginInstallDialog = false;
bUpdateMode = false;
--Dialog.Message("DEBUG", "  Offer Plug-in Install: Script Started. ", MB_OK, MB_ICONNONE, MB_DEFBUTTON1);
fDebug("  Offer Plug-in Install: Script Started. ");

--[[ **** FUNCTION DESCRIPTION: IsPrintServerInstalled ****
If the Print Server is installed and reqistered, the following statement will return the full path
and file name of the capture server dll.
]]--
function IsPrintServerInstalled()
	PrintServerDLL = Registry.GetValue(0, "CLSID\\{39826198-6E77-4D72-8757-FDFAF628AE85}\\InprocServer32","",false);
	LastError = Application.GetLastError();
	if(LastError ~= 0)then 	-- Registry value was not found. Print Server was not detected.	
		return false;
	else -- Registry entry was found.  Confirm that the Print Server DLL is on the system where it says it is.
		if(File.DoesExist(PrintServerDLL)) then
			fDebug("  Offer Plug-in Install: Print Server found. ");
			return true;
		else
			fDebug("  Offer Plug-in Install: Print Server not found. ");
			return false;
		end
	end
end

--[[ **** FUNCTION DESCRIPTION: IsVersion5OrNewer ****
Print Server has been detected.  Get version information from the pritn server dll.
and use it to determine if the print server meets the minimum v5 requirement.
]]--
function IsVersion5OrNewer()
	local PrintServerInfo = File.GetVersionInfo(PrintServerDLL);	
	-- Extract the build number from the PrintServer DLL.
	-- The DLL version info is of the form BuildNumber.month.day.year
	local BuildNumberDelimiter = String.Find(PrintServerInfo.FileVersion,".",1,false); -- look for the first decimal point. The build number will be to the left of this.
	if (BuildNumberDelimiter < 4) then
		-- build number should be at least three digits.  If it isn't, there is something wrong with the version string
		return false;
	end
	local BuildNumberString = String.Left((PrintServerInfo.FileVersion),(BuildNumberDelimiter-1)); -- extract the build number from the version string.
	local BuildNumber = String.ToNumber(BuildNumberString);
	if (BuildNumber > 481) then -- 5.0 build number was 482
		fDebug("  Offer Plug-in Install: Print Server version OK. Build Number: "..BuildNumberString);
		return true;
	else
		-- build number is less that v5.  Print server too old.
		fDebug("  Offer Plug-in Install: Print Server too old. Build Number: "..BuildNumberString);
		return false;
	end
end

--[[ **** FUNCTION DESCRIPTION: GetPrintServerDLLNameAndLocation ****
Print Server has been detected and its path is held in the variable PrintServerDLL.  
Parse out the folder path so it can be used to look for the plug-in dll.
]]--
function GetPrintServerDLLNameAndLocation()
	local SplitApartPath = String.SplitPath(PrintServerDLL);
	local DriveLetter = SplitApartPath.Drive;
	local Folder = SplitApartPath.Folder;
	PrintServerPath = String.Concat(DriveLetter, Folder); -- this is the path to the folder containing the Print Server DLL.
	fDebug("  Offer Plug-in Install: Print Server folder: "..PrintServerPath);
	return true;
end

--[[ **** FUNCTION DESCRIPTION: IsPluginInstalled ****
The Print Server has been detected.  Determine if the XPS Card Printer Plug-in exists
in the Print Server directory.
]]--
function IsPluginInstalled()
	if (File.DoesExist(PrintServerPath.."\\psxpscardprinterplugin.dll")) then
		fDebug("  Offer Plug-in Install: Plug-in found on system. ");
		return true;
	else
		fDebug("  Offer Plug-in Install: Plug-in not found on system. ");
		return false;
	end
end


--[[ **** FUNCTION DESCRIPTION: IsInstalledPluginOlder ****
The plugin has been detected. Get its version and parse the version string to pull out the first chunk.
This is the build number.  It will always increment so we can use it to compare the installed version
of the plugin to the version on the installation media to determine which is the most recent build.
]]--
function IsInstalledPluginOlder()
	local InstalledPluginInfo = File.GetVersionInfo(PrintServerPath.."\\psxpscardprinterplugin.dll");	
	-- Extract the build number from the DLL.
	-- The DLL version info is of the form BuildNumber.month.day.year
	local BuildNumberDelimiter = String.Find(InstalledPluginInfo.FileVersion, ".", 1, false); -- look for the first decimal point. The build number will be to the left of this.
	if (BuildNumberDelimiter < 4) then
		-- build number should be at least three digits.  If it isn't, there is something wrong with the version string
		return false;
	end
	local BuildNumberString = String.Left((InstalledPluginInfo.FileVersion),(BuildNumberDelimiter-1)); -- extract the build number from the version string.
	fDebug("  Installed Plug-in Build Number: "..BuildNumberString.."  Media Plug-in Build Number: "..MediaPluginBuildNmbr);
	local InstalledPluginBuildNumber = String.ToNumber(BuildNumberString);
	-- get build number of plug-in on the install media.
	
	if (InstalledPluginBuildNumber < MediaPluginBuildNmbr) then -- plugin is older.
		return true;
	else
		-- build number the same or newer. Do not offer to install plug-in
		return false;
	end
end



--[[
This code calls the above functions to determine the Print Server info.
]]--
if (IsPrintServerInstalled()) then
	if (IsVersion5OrNewer()) then
		if (GetPrintServerDLLNameAndLocation()) then
			if (IsPluginInstalled()) then
				if (IsInstalledPluginOlder()) then
					bDisplayPluginInstallDialog = true;
					bUpdateMode = true;
				else -- plug-in installed and same or newer version
					bDisplayPluginInstallDialog = false;
					bUpdateMode = false;
				end
			else -- plug-in is not installed. Set the flag to show the dialog.
				bDisplayPluginInstallDialog = true;
				bUpdateMode = false;
			end
		else -- print server path lookup failed
			bDisplayPluginInstallDialog = false;
			bUpdateMode = false;
		end
	else -- print server too old
		bDisplayPluginInstallDialog = false;
		bUpdateMode = false;
	end	
else -- print server not installed
	bDisplayPluginInstallDialog = false;
	bUpdateMode = false;
end